/*
    DataViewer visualization package
    Copyright (C) 1997 Randy Paffenroth and Thomas Stone

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
*/

#include <iostream.h> 
#include <fstream.h>
#include <strstream.h>
#include <math.h>
#include "earth.h"
#include "DV.h"

int earth(DVcreate_image_arguments &dv)
{ 
  // Create the sun as an orange sphere of radius 1, centered in the origin.
  DVspheres sun(0, 0, 0, 0.8);
  sun.color[0]=DVcolor(1, 0.5, 0);
  dv.top.copy_child(sun);

  Animate_earth(dv);

  dv.top.geometry_complexity = 9;

  // Set the frames to be drawn.
  dv.dvhints.set_animation_first_frame_number(0); 
  dv.dvhints.set_animation_last_frame_number(99); 

  return 1;
}

// This function creates an animation based on some spheres with positions
// following an ellipse around the origin
int Animate_earth(DVcreate_image_arguments &dv)
{
  float x, y, arc=M_PI/25;

  // An animated container that will contain the Earth at different positions
  DVselection_container earth_move;
  // Set the container's color to blue, so we don't have to set it for each
  // sphere
  earth_move.color[0]=DVcolor(0.4, 0.4, 1);

  for (int i=0; i<100; i++) {
    x = 6 * cos(i*arc);
    y = 9 * sin(i*arc);

    // Make a sphere with center on the ellipse of axis 6 and 9, in the (x,y)
    // plane, centered in the origin.
    DVspheres earth(x, y, 0, 0.45);
    earth_move.copy_child(earth);
  }

  dv.top.copy_child(earth_move);
}

