/*
    DataViewer visualization package
    Copyright (C) 1997 Randy Paffenroth and Thomas Stone

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
*/

#include <iostream.h> 
#include <fstream.h>
#include <strstream.h>
#include <math.h>
#include "DV.h"
#include "DVrotation_interpolation_container.h"
#include "DVtranslation_interpolation_container.h"
#include "walk.h"

// Creates a tree representing a man and animates it to simulate 
// a walking motion.
int walk(DVcreate_image_arguments &dv)
{ 
  // Create a reference plane
  DVboxes plane(-10, 0, -10, 10, 0, 10);
  plane.color[0]=DVcolor(1.0, 1.0, 0.0);
  dv.top.copy_child(plane);

  cout << "plane" << endl;
  // Create a walking path
  LCVMarray_2D<float> path(2,3);
  path[0][0]= -8;
  path[0][1]= 0.0;
  path[0][2]= -8;
  path[1][0]= -8;
  path[1][1]= 0.0;
  path[1][2]= 8;
  DVlines path_line(path);
  path_line.color[0]=DVcolor(1.0, 0.0, 0.0);
  path_line.set_line_status(DV_CYLINDER);
  path_line.width[0]=0.1;
  dv.top.copy_child(path_line);

  //cout << "path" << endl;
  DVtranslation_interpolation_container walk;
  LCVMarray<int> frames(2);
  frames[0]=0;
  frames[1]=69;
  LCVMarray_2D<float> vectors(2, 3);
  vectors[0][0]=-8.0;
  vectors[0][1]=0.0;
  vectors[0][2]=8.0;
  vectors[1][0]=-8.0;
  vectors[1][1]=0.0;
  vectors[1][2]=-8.0;
  walk.set_data(frames, vectors);
  //cout << "walk" << endl;

  Animate_man_walk(walk, 3, 3); 
  //cout << "man" << endl; 
  dv.top.copy_child(walk);

  dv.top.geometry_complexity=9;
  dv.top.color[0]=DVcolor(0.4, 0.4, 1.0);

  // Set the frames to be drawn.
  dv.dvhints.set_animation_first_frame_number(0); 
  dv.dvhints.set_animation_last_frame_number(69); 

  return 1;
}

// This animates a simple model of a man walking.
int Animate_man_walk(DVcontainer &dv_top, int nr_cycles, int step)
{
  DVrotation_interpolation_container center;
  Make_leg(center, -0.8, 0, nr_cycles, step);
  Make_leg(center, 0.8, 3, nr_cycles, step);

  DVrotation_interpolation_container head_joint;
  DVspheres head(0.0, 0.0, 0.0, 1.2);
  head.set_translate(0.0, 1.1, 0.0);
  head_joint.copy_child(head);
  DVcontainer head_trans;
  head_trans.copy_child(head_joint);

  DVrotation_interpolation_container neck;
  Make_part(neck, 0.6, 1);
  head_trans.set_translate(0.0, 0.6, 0.0);
  neck.copy_child(head_trans);
  //cout << "neck: " << neck << endl;
  Make_arm(neck, -1.5);
  Make_arm(neck, 1.5);
  DVcontainer neck_trans;
  neck_trans.copy_child(neck);

  DVrotation_interpolation_container spine;
  Make_part(spine, 2.7, 1);
  neck_trans.set_translate(0.0, 2.7, 0.0);
  spine.copy_child(neck_trans);
  DVcontainer spine_trans;
  spine_trans.copy_child(spine);
  
  Make_part(center, 1.0, 1);
  spine_trans.set_translate(0.0, 1.0, 0.0);
  center.copy_child(spine_trans);

  center.set_translate(0.0, 6.0, 0.0);
  dv_top.copy_child(center);
}

// Makes a leg and attaches it to the center. It translates the entire
// leg by the given distance horizontally. The angle_start is used for
// the left - right rotations.
void Make_leg(DVrotation_interpolation_container &center,
              float distance, int angle_start, int nr_cycles, int step)
{
  float walk_angles[6][8] =
     {{0.0, -15.0, -15.0, 30.0, 45.0, 30.0, 15.0, 7.0},
      {0.0, -15.0, -45.0, -75.0, -60.0, -30.0, -15.0, -7.0},
      {0.0, -30.0, 30.0, 15.0, 0.0, 15.0, 0.0, 1.0},
      {45.0, 30.0, 15.0, 7.0, 0.0, -15.0, -15.0, 30.0},
      {-60.0, -30.0, -15.0, -7.0, 0.0, -15.0, -45.0, -75.0},
      { 0.0, 15.0, 0.0, 1.0, 0.0, -30.0, 30.0, 15.0}};

  DVrotation_interpolation_container ankle;
  Make_part(ankle, -1.5, 2);
  Add_rotation(ankle, walk_angles[angle_start+2], 8, step, nr_cycles);
  DVcontainer ankle_trans;
  ankle_trans.copy_child(ankle);

  DVrotation_interpolation_container knee;
  Make_part(knee, -2.8, 1);
  ankle_trans.set_translate(0.0, -2.8, 0.0);
  knee.copy_child(ankle_trans);
  Add_rotation(knee, walk_angles[angle_start+1], 8, step, nr_cycles);
  DVcontainer knee_trans;
  knee_trans.copy_child(knee);

  DVrotation_interpolation_container hips;
  Make_part(hips, -2.7, 1);
  knee_trans.set_translate(0.0, -2.7, 0.0);
  hips.copy_child(knee_trans);
  Add_rotation(hips, walk_angles[angle_start+0], 8, step, nr_cycles);
  DVcontainer hips_trans;
  hips_trans.copy_child(hips);

  hips_trans.set_translate(distance, 0.0, 0.0);
  center.copy_child(hips_trans);
  Make_part(center, distance, 0);
}

// Makes an arm at attaches it to the neck at the given horizontal distance
void Make_arm(DVcontainer &neck, float distance)
{
  DVrotation_interpolation_container wrist;
  Make_part(wrist, -1.2, 1);
  DVcontainer wrist_trans;
  wrist_trans.copy_child(wrist);

  DVrotation_interpolation_container elbow;
  Make_part(elbow, -1.9, 1);
  wrist_trans.set_translate(0.0, -1.9, 0.0);
  elbow.copy_child(wrist_trans);
  DVcontainer elbow_trans;
  elbow_trans.copy_child(elbow);

  DVrotation_interpolation_container shoulder;
  Make_part(shoulder, -2.3, 1);
  elbow_trans.set_translate(0.0, -2.3, 0.0);
  shoulder.copy_child(elbow_trans);
  DVcontainer shoulder_trans;
  shoulder_trans.copy_child(shoulder);

  shoulder_trans.set_translate(distance, 0.0, 0.0);
  neck.copy_child(shoulder_trans);
  Make_part(neck, distance, 0);
}

// Creates a body part of a given length in the given direction
// and attaches it to the joint
void Make_part(DVcontainer &joint, double length, int which_coord)
{
  DVspheres joint_sph(0.0, 0.0, 0.0, 0.2);
  LCVMarray_2D<float> points(2, 3);
  for (int i=0; i<3; i++) {
    points[0][i]=0.0;
    points[1][i]=0.0;
  }
  points[1][which_coord]=length;

  DVlines part(points);
  part.width[0]=0.15;
  part.set_line_status(DV_CYLINDER);
  joint.copy_child(joint_sph);
  joint.copy_child(part);
}

// This function adds a number of key rotations to the joint
void Add_rotation(DVrotation_interpolation_container &joint,
                  float *angles, int nr_angles, int step, int nr_cycles)
{
  int nr_key_frames = nr_angles*nr_cycles, i;
  cout << "nr frames: " << nr_key_frames << endl;

  // Set the frames
  LCVMarray<int> the_frames(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_frames[i] = i * step;

  // Set the rotation axes : constant for all the frames, around Oz
  LCVMarray_2D<float> the_axes(nr_key_frames, 3);
  for (i=0; i<nr_key_frames; i++) {
    the_axes[i][0] = 50.0;
    the_axes[i][1] = 0.0;
    the_axes[i][2] = 0.0;
  }

  // Set the rotation angles by cycling the parameter
  LCVMarray<float> the_angles(nr_key_frames);
  for (i=0; i<nr_key_frames; i++)
    the_angles[i] = angles[i%nr_angles];

  // Send all this data to the container
  joint.set_data(the_frames, the_axes, the_angles);
}

